TaskManagerData.collapsed = {};

TaskManagerData.minimappos = 55;
TaskManagerData.hideDone = false;
TaskManagerData.orderBy = 1;

TaskManagerData.selectedId = nil;

local ORDER_BY_VALUES = {
    "Order by Category",
    "Order by Location",
}

function TaskManager_OnLoad(self)
    self:RegisterEvent("ADDON_LOADED");
    self:RegisterEvent("PLAYER_ENTERING_WORLD");
    
    SLASH_NODUEL1 = "/tm";
    SlashCmdList["TASKMANAGER"] = function(msg)
        TaskManager_SlashCommandHandler(msg);
    end

    TaskManager_CheckButtonVisibility();
    
    TaskManager_InitLinks();
end

local function removeTextures(frame)
    for i, child in ipairs({frame:GetRegions()}) do

        if (child and child:IsObjectType("texture")) then
            child:SetTexture(nil);
        end
    end
end

function TaskManager_OnEvent(self, event, ...)
    if (event == "ADDON_LOADED") then

        local addonName = ...;
        if (addonName == "TaskManager") then
            if (not TaskManagerData.minimappos) then
                TaskManagerData.minimappos = 55;
            end

            TaskManager_MinimapButton_Reposition();
            TaskManager_CheckButtonVisibility();
            
            TaskManager_StartDelay(1, TaskManager_Init);
            TaskManager_StartDelay(3, TaskManager_CheckTasks);
        end
    end
    
    removeTextures(TaskManagerFrameOrderByDropDown);
end

function TaskManager_CheckTasks()
    local guid = UnitGUID("player");
    local entries = TaskManagerConfig_GetOpenTasks(guid);
    
    if (not entries or #entries == 0) then
        return;
    end
    
    DEFAULT_CHAT_FRAME:AddMessage("|c333399ffTaskManager:|r There are some unfinished tasks:", 1.0, 1.0, 1.0, 1, 10);
    
    local title;
    
    for index=1, #entries do
        if (not entries[index].isHeader) then 
            title = entries[index].title; 
            DEFAULT_CHAT_FRAME:AddMessage("|c333399ff  - |r "..title, 1.0, 1.0, 1.0, 1, 10);
        end
    end
end

function TaskManager_InitLinks()
    QuestMapFrame.DetailsFrame.BackButton:SetWidth(60);
    
    QuestMapFrame.DetailsFrame.TaskButton = CreateFrame("Button", nil, QuestMapFrame.DetailsFrame);
    QuestMapFrame.DetailsFrame.TaskButton:SetSize(16, 16);
    QuestMapFrame.DetailsFrame.TaskButton:SetPoint("LEFT", QuestMapFrame.DetailsFrame.BackButton, "RIGHT", 3, 0);
    QuestMapFrame.DetailsFrame.TaskButton:SetNormalTexture("Interface\\AddOns\\TaskManager\\icons\\task_icon");
    QuestMapFrame.DetailsFrame.TaskButton:SetPushedTexture("Interface\\AddOns\\TaskManager\\icons\\task_icon");
    QuestMapFrame.DetailsFrame.TaskButton:SetHighlightTexture("Interface\\AddOns\\TaskManager\\icons\\task_icon");
    QuestMapFrame.DetailsFrame.TaskButton:HookScript("OnEnter", TaskManager_ShowButtonTooltip);
    QuestMapFrame.DetailsFrame.TaskButton:HookScript("OnLeave", TaskManager_HideButtonTooltip);
    QuestMapFrame.DetailsFrame.TaskButton:HookScript("OnClick", function()
                        PlaySound("gsCharacterSelectionCreateNew");
                        
                        local questLogSelection = GetQuestLogSelection();
                        local title, level, _, _, _, _, frequency = GetQuestLogTitle(questLogSelection);
                        -- frequency: 1 = single, 2 = daily, 3 = weekly
                        
                        local questDescription, questObjectives = GetQuestLogQuestText();
                        
                        TaskManagerConfig_CreateTask(title, "Quest", questObjectives, frequency, 2);
                    end);
    QuestMapFrame.DetailsFrame.TaskButton.tooltip = "create task";
end

function TaskManager_ShowButtonTooltip(self)
    GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
    GameTooltip:SetText(self.tooltip);
    GameTooltip:Show();
end

function TaskManager_HideButtonTooltip()
    GameTooltip:FadeOut();
end

function TaskManager_ReinitSettings()
    UIDropDownMenu_SetText(TaskManagerFrameOrderByDropDown, ORDER_BY_VALUES[TaskManagerData.orderBy]);
    TaskManager_TasksOnUpdate();
end

function TaskManager_OrderByDropDown_Initialize()
    for index, data in ipairs(ORDER_BY_VALUES) do
        local info = UIDropDownMenu_CreateInfo();
        info.text = data;
        info.value = index;
        info.func = function()
            TaskManager_OrderBySelect(index);
        end
        UIDropDownMenu_AddButton(info);
    end
end

function TaskManager_OrderBySelect(index)
    TaskManagerData.orderBy = index;
    TaskManagerData.selectedId = nil;
    TaskManager_ReinitSettings();
end

function TaskManager_SlashCommandHandler(msg)
    if (msg == "reload") then
        ReloadUI();
    end
end

function TaskManager_MinimapButton_DraggingFrame_OnUpdate()

    local xpos,ypos = GetCursorPosition();
    local xmin,ymin = Minimap:GetLeft(), Minimap:GetBottom();

    xpos = xmin-xpos/UIParent:GetScale()+70;
    ypos = ypos/UIParent:GetScale()-ymin-70;

    TaskManagerData.minimappos = math.deg(math.atan2(ypos,xpos));
    TaskManager_MinimapButton_Reposition();
end

function TaskManager_MinimapButton_Reposition()
    TaskManager_MinimapButton:SetPoint("TOPLEFT", "Minimap", "TOPLEFT",
        52-(80*cos(TaskManagerData.minimappos)), (80*sin(TaskManagerData.minimappos))-52);
end

function TaskManager_CheckButtonVisibility()
    if (TaskManagerData.hideButton) then
        TaskManager_MinimapButton:Hide();
    else
        TaskManager_MinimapButton:Show();
    end
end

function TaskManager_ToggleButton()
    if (TaskManagerData.hideButton) then
        DEFAULT_CHAT_FRAME:AddMessage("TaskManager: show minimap button", 1.0, 1.0, 1.0, 1, 10);
        TaskManagerData.hideButton = false;
    else
        DEFAULT_CHAT_FRAME:AddMessage("TaskManager: hide minimap button", 1.0, 1.0, 1.0, 1, 10);
        TaskManagerData.hideButton = true;
    end

    TaskManagerData_CheckButtonVisibility();
end

function TaskManager_Init()
    TaskManagerConfig_InitData();
    
    TaskManagerFrameCheckBoxHideDone:SetChecked(TaskManagerData.hideDone);
    
    local name = UnitName("player");
    TaskManagerFrame.title:SetText(name.."'s Tasks");
    
    TaskManager_ReinitSettings();
end

function TaskManager_MouseOver(button)
    TaskManager_ShowTooltip(button);
end

function TaskManager_MouseOut(button)
    GameTooltip:FadeOut();
end

function TaskManager_ShowTooltip(button)
    GameTooltip_SetDefaultAnchor(GameTooltip, button);
    TaskManager_CreateTooltip(GameTooltip);
end

function TaskManager_CreateTooltip(tooltip)
    tooltip:SetText("|cffffffffTaskManager|r");
    
    tooltip:Show();
end

function TaskManager_MinimapButton_OnClick()
    if (TaskManagerFrame:IsVisible()) then
        TaskManagerFrame:Hide();
    else
        TaskManagerFrame:Show();
    end
end

function TaskManagerFrame_OnMouseDown(frame, button)
    if (button == "LeftButton") then
        TaskManagerFrame:StartMoving();
    end
end

function TaskManagerFrame_OnMouseUp(frame, button)
    TaskManagerFrame:StopMovingOrSizing();
end

function TaskManager_Tasks_OnLoad(self)
    self:RegisterForClicks("LeftButtonUp", "RightButtonUp");
end

function TaskManager_Tasks_OnEnter(self)
    if (self.isHeader) then
        GameTooltip:FadeOut();
        return;
    end

    GameTooltip_SetDefaultAnchor(GameTooltip, self);
    GameTooltip:SetOwner(self, "ANCHOR_RIGHT");

    GameTooltip:SetText(self.data.title, 1, 1, 1, true);
    GameTooltip:AddLine(self.data.category);
    
    GameTooltip:AddLine(" ");    
    GameTooltip:AddLine(self.data.description, 1, 1, 1, true);
    
    GameTooltip:AddLine(" ");    
    GameTooltip:AddDoubleLine(HIGHLIGHT_FONT_COLOR_CODE.."What:"..FONT_COLOR_CODE_CLOSE, TaskManager_GetTypeString(self.data.type)..", "..TaskManager_GetRepeatString(self.data.refreshRate));
    GameTooltip:AddDoubleLine(HIGHLIGHT_FONT_COLOR_CODE.."Where:"..FONT_COLOR_CODE_CLOSE, self.data.location);  
    
    GameTooltip:AddLine(" ");  
    
    GameTooltip:AddLine(HIGHLIGHT_FONT_COLOR_CODE.."Who:"..FONT_COLOR_CODE_CLOSE);  
    
    local guid;
    local actCharacter;
    local done;
    
    local name, realm;
    
    for i=1, #self.data.characters do
        guid = self.data.characters[i];
        
        actCharacter = TaskManagerData.characters[guid];
    
        local classColor = RAID_CLASS_COLORS[actCharacter.englishClass];
        local classColorText = string.format("ff%.2x%.2x%.2x", classColor.r * 255, classColor.g * 255, classColor.b * 255);
        
        if (tContains(self.data.done, guid)) then
            done = DONE;
        else
            done = "";
        end
        
        name = actCharacter.name;
        realm = GetRealmName();
        
        if (realm) then
            if (actCharacter.realm) then
                if (realm ~= actCharacter.realm) then
                    name = name.."@"..actCharacter.realm;
                end
            end
        end
        
        GameTooltip:AddDoubleLine(" - |c"..classColorText..name.." |r", done);
    end

    GameTooltip:Show();
end

function TaskManager_Tasks_OnLeave(self)
    GameTooltip:FadeOut();
end

function TaskManager_Tasks_OnClick(self, button)    
    if (self.isHeader) then
        TaskManager_ToggleCategory(self.category);
        return;
    end
    
    if (not TaskManagerConfigFrame:IsVisible()) then
        return;
    end
    
    TaskManagerData.selectedId = self.data.id;
    TaskManagerConfig_ToggleEnabled();
    
    TaskManager_TasksOnUpdate();
end

function TaskManagerFrame_ScrollFrame_OnLoad(self)
    HybridScrollFrame_OnLoad(self);
    self.update = TaskManager_TasksOnUpdate;
    HybridScrollFrame_CreateButtons(self, "TaskManagerFrameTaskTemplate");
end

function TaskManager_OnShow()
    TaskManager_ResetRepeatableTasks();
    TaskManager_TasksOnUpdate();
end

function TaskManager_ResetRepeatableTasks()
    local now = date("*t"); -- { hour=1, min=31, sec=41, wday=1, day=14, month=6, year=2009, yday=165, isdst=true, ct=123123123123 }
    now.ct = time();

    if (TaskManagerData.lastCheck) then
        -- berechne Abstand zu 3 Uhr am selben Tag wie 'to'
        local secondsOfDay = now.hour * 60 * 60 + now.min * 60 + now.sec;
        local diffTo = secondsOfDay - 3 * 60 * 60;

        -- wenn 'to' vor 3 Uhr, dann muss 3 Uhr vom Vortag geprüft werden
        if (diffTo < 0) then
            diffTo = diffTo + 24 * 60 * 60;
        end

        if (TaskManagerData.lastCheck.ct < now.ct - diffTo) then
            TaskManager_ResetRepeatable(2); -- daily
        end

        -- Suche den letzten Mittwoch 3am.
        diffTo = secondsOfDay - 3 * 60 * 60;

        local mi0 = mod(now.wday + 3, 7);

        if (diffTo < 0 and mi0 == 0) then
            mi0 = 7;
        end

        diffTo = diffTo + mi0 * 24 * 60 * 60;

        if (TaskManagerData.lastCheck.ct < now.ct - diffTo) then
            TaskManager_ResetRepeatable(3); -- weekly
        end  
    end
    
    TaskManagerData.lastCheck = now;
end

function TaskManager_ResetRepeatable(refreshRate) -- 1 = none, 2 = daily, 3 = weekly
    for index, data in ipairs(TaskManagerData.tasks) do
        if (data.refreshRate == refreshRate) then
            tm_debug("TaskManager: Task "..data.title.." was reset.");
            data.done = {};
        end
    end
end

function TaskManager_TasksOnUpdate()
    if ( not TaskManagerFrame:IsShown() ) then
        return;
    end
    
    local guid = UnitGUID("player");
    
    if (TaskManagerConfigFrame:IsVisible()) then
        guid = nil;
    end

    local entries = TaskManagerConfig_GetAllTasks(guid);

    local numEntries = #entries;

    -- hide the highlight frame initially, it may be shown when we loop through the quest listing if a quest is selected
    TaskManagerFrameHighlightFrame:Hide();

    -- Update the quest listing
    local buttons = TaskManagerFrameScrollFrame.buttons;
    local numButtons = #buttons;
    local scrollOffset = HybridScrollFrame_GetOffset(TaskManagerFrameScrollFrame);
    local buttonHeight = buttons[1]:GetHeight();
    local displayedHeight = 0;

    local title, done;
    for i=1, numButtons do

        local index = i + scrollOffset;
        local task = buttons[i];
        
        task.expandText:SetText("");
        task.countInfo:SetText("");
        task.typeInfo:SetText("");

        if ( index <= numEntries ) then
                
            if (entries[index].isHeader) then                   
                task.isHeader = true;
                             
                title = entries[index].title;       
                
                if (not title or title == "") then                
                    if (TaskManagerData.orderBy == 1) then -- order by category
                        title = "No category";
                    elseif (TaskManagerData.orderBy == 2) then -- order by location
                        title = "No location";
                    else -- default
                        title = "No category";
                    end
                end
                
                        
                task.category = title;
                
                task:Show();
                task:SetText(title);
                
                task.expandText:SetText("-");
                
                if (tContains(TaskManagerData.collapsed, title)) then
                    task.expandText:SetText("+");
                end
                
                task.countInfo:SetText("["..entries[index].countTasks.."]");
                
                task.normalText:SetPoint("LEFT", task, "LEFT", 20, 0);
                
                task:SetNormalFontObject("GameFontHeader");
                task:SetHighlightFontObject("GameFontHeader");
                task:SetDisabledFontObject("GameFontHeader");
                
                task.background:Show();

                task.checkbox:Hide();
            else                      
                task.isHeader = false;   
                       
                title = entries[index].title;
                
                task.data = entries[index];
                task.checkbox.data = entries[index];

                task:Show();
                task:SetText(title);
                
                -- Info
                task.typeInfo:SetText(TaskManager_GetTypeString(task.data.type));
                
                -- set unique taks id
                task.id = index;
                
                task.normalText:SetPoint("LEFT", task, "LEFT", 35, 8);
                
                task:SetNormalFontObject("GameFontTask");
                task:SetHighlightFontObject("GameFontTask");
                task:SetDisabledFontObject("GameFontTask");

                task.background:Hide();

                task.checkbox:Show();
                
                done = tContains(entries[index].done, UnitGUID("player"));
                TaskManager_SetChecked(task.checkbox, done);

                -- this isn't a header, hide the header textures
                task:SetNormalTexture("");
                task:SetHighlightTexture("");

                if (TaskManagerConfigFrame:IsVisible() and entries[index].id == TaskManagerData.selectedId) then
                    -- reposition highlight frames
                    TaskManagerFrameHighlightFrame:SetParent(task);
                    TaskManagerFrameHighlightFrame:SetPoint("TOPLEFT", task, "TOPLEFT", 0, 0);
                    TaskManagerFrameHighlightFrame:SetPoint("BOTTOMRIGHT", task, "BOTTOMRIGHT", 0, 0);
                    TaskManagerFrameHighlightFrame:Show();
                end
            end
        else
            task:Hide();
        end

        displayedHeight = displayedHeight + buttonHeight;
    end

    HybridScrollFrame_Update(TaskManagerFrameScrollFrame, numEntries * buttonHeight, displayedHeight);
end

function TaskManager_SetChecked(checkbox, checked)
    if (checked) then
        checkbox:SetNormalTexture("Interface\\AddOns\\TaskManager\\icons\\checkbox_checked");
        checkbox:SetHighlightTexture("Interface\\AddOns\\TaskManager\\icons\\checkbox_checked_hover");
    else
        checkbox:SetNormalTexture("Interface\\AddOns\\TaskManager\\icons\\checkbox");
        checkbox:SetHighlightTexture("Interface\\AddOns\\TaskManager\\icons\\checkbox_hover");
    end
    
    checkbox:SetPushedTexture(nil);
    checkbox:SetDisabledTexture(nil);
end

function TaskManager_ToggleDone(checkbox)

    if (checkbox.data.isAccount) then
        TaskManager_ToggleAccountDone(checkbox);
        TaskManager_TasksOnUpdate();
        
        GameTooltip:Hide();
        TaskManager_Tasks_OnEnter(checkbox:GetParent());
        return;
    end

    local guid = UnitGUID("player");
    if (tContains(checkbox.data.done, guid)) then
        for index, foundGuid in ipairs(checkbox.data.done) do
            if (foundGuid == guid) then
                tremove(checkbox.data.done, index);
            end
        end
    else
        tinsert(checkbox.data.done, guid);
    end
    
        
    GameTooltip:Hide();
    TaskManager_Tasks_OnEnter(checkbox:GetParent());
    TaskManager_TasksOnUpdate();
end

function TaskManager_ToggleAccountDone(checkbox)
    if (#checkbox.data.done > 0) then
        checkbox.data.done = {};
    else
        checkbox.data.done = {};
        
        local guid;
        for i=1, #checkbox.data.characters do
            guid = checkbox.data.characters[i];
            tinsert(checkbox.data.done, guid);
        end
    end
end

function TaskManager_ToggleCategory(category)
    if (tContains(TaskManagerData.collapsed, category)) then
        TaskManager_ExpandCategory(category);
    else
        TaskManager_CollapseCategory(category);
    end
    
    TaskManager_TasksOnUpdate();
end

function TaskManager_ExpandCategory(category)
    for index, foundCategory in ipairs(TaskManagerData.collapsed) do
        if (foundCategory == category) then
            tremove(TaskManagerData.collapsed, index);
        end
    end
end

function TaskManager_CollapseCategory(category)
    tinsert(TaskManagerData.collapsed, category);
end

function TaskManager_ToggleConfig()
    if (TaskManagerConfigFrame:IsVisible()) then
        TaskManagerConfigFrame:Hide();
    else
        TaskManagerConfigFrame:Show();
    end
end
